import os

import constants
from download_shared_file import download_file_from_google_drive
from prepare_fixja_properties_file import generate_new_properties_file_from_old
from read_fj_output import read_new_fixja_result
from utils import unzip


# =====================================================================================================
# Operation Function Definitions
# =====================================================================================================
def show_config(exp_config):
    print("Config: ", str(exp_config))


def prepare_all_d4j(exp_config):
    for candidate in constants.D4J_CANDIDATES.keys():
        repo_path = os.path.join(exp_config.get_working_path(), candidate)
        prepare_d4j_bv(exp_config, repo_path)


def download_all_d4j(exp_config):
    for can_key in constants.D4J_CANDIDATES.keys():
        zip_path = os.path.join(exp_config.get_working_path(), can_key + constants.ZIP_SUFFIX)
        if not os.path.exists(zip_path):
            download_file_from_google_drive(constants.D4J_CANDIDATES[can_key], zip_path)


def download_if_not_exist(repository_path):
    if not os.path.exists(repository_path):
        zip_repo_path = repository_path + constants.ZIP_SUFFIX
        if not os.path.exists(zip_repo_path):
            # download the specific project and store in 'zip_repo_path'
            zip_file = os.path.split(zip_repo_path)
            zip_file_name = zip_file[1][0:len(zip_file[1]) - len(constants.ZIP_SUFFIX)]
            if zip_file_name in constants.D4J_CANDIDATES.keys():
                download_file_from_google_drive(constants.D4J_CANDIDATES[zip_file_name], zip_repo_path)
            elif zip_file_name in constants.D4J_OLD_CANDIDATES.keys():
                download_file_from_google_drive(constants.D4J_OLD_CANDIDATES[zip_file_name], zip_repo_path)
            else:
                print('download ' + zip_file_name + ' fail.')
                return
        unzip(zip_repo_path, repository_path)


def prepare_d4j_bv(exp_config, repository_path):
    drive, repo = os.path.split(repository_path)
    download_if_not_exist(repository_path)
    property_file_path = generate_new_properties_file_from_old(repository_path, exp_config)
    print('The buggy program ' + repo + ' is ready. Program arguments of JAID running configuration is:')
    print(constants.FIXJA_ARG_PROPERTY + ' ' + property_file_path)
    return property_file_path


def read_d4j_bv(repository_path):
    for inner_inner_folder in os.listdir(repository_path):
        if inner_inner_folder == 'fixja_output':  # find the experiment output in repo
            fixja_output_dir = os.path.join(repository_path, inner_inner_folder)
            read_new_fixja_result(fixja_output_dir)


def run_(exp_config, properties_file_path):
    command = repr(os.path.normpath(os.path.join(exp_config.JDKDir, 'bin', 'java'))).replace('\'',
                                                                                             '"') + ' -jar ' + \
              exp_config.get_jaid_path() + ' ' + constants.FIXJA_ARG_PROPERTY + ' ' + properties_file_path
    print('EXE COMMAND: ' + command)
    os.system(command)


def get_repo_path(exp_config, repo_name, bug_id):
    repo = constants.repositories[repo_name.lower()] + bug_id
    return repo, os.path.join(exp_config.get_working_path(), repo)


def print_available_bv():
    print('All available projects and buggy ids:')
    print('<REPO_NAME>: <BUG_VERSION_1>,<BUG_VERSION2>,...')
    for project in constants.valid_bvs.keys():
        print(project + ' : ' + ','.join(sorted(set(constants.valid_bvs[project]))))
    print('=' * 10)
    print('Defects4J <BUG_ID> format:' + constants.D4J_BUGID_FORMAT+' (e.g.,Lang33)')
    print('=' * 10)
    print('Example for running JAID to fix a buggy program:')
    print('$ python3 <PATH_TO_jaid_exp_pre>/src/main.py ' + constants.COMMAND_RUN + ' Lang33,math80')
    # print('All available IntroClassJava repositories:')
    print('Example for reading JAID\'s output of fixing a buggy program:')
    print('$ python3 <PATH_TO_jaid_exp_pre>/src/main.py ' + constants.COMMAND_READ + ' Lang33,math80')
    print('\n')
