#! /usr/bin/env python3
import argparse
import os

import constants
from config import get_script_root_path
from utils import init_env, read_config_file, split_src_num

init_env(os.path.join(get_script_root_path(), constants.REQUIREMENT_FILE_NAME))

from fuctions import print_available_bv, show_config, get_repo_path, prepare_d4j_bv, read_d4j_bv, run_
from read_fj_output import select_desired_fixaction, select_opposite_snapshot



def add_command(com_group, command_key):
    com_group.add_argument(command_key, action='store_true',
                           help=constants.Commands[command_key], required=False)


parser = argparse.ArgumentParser(description='Script for JAID experiment buggy repositories.\n')

exp_com = parser.add_argument_group('Experiment actions')
add_command(exp_com, constants.COMMAND_SHOW_INFO)
add_command(exp_com, constants.COMMAND_SHOW_CONFIG)

exp_com.add_argument(constants.COMMAND_PREPARE,
                     metavar=constants.ARGS_FORMAT,
                     help=constants.Commands[constants.COMMAND_PREPARE], required=False)
exp_com.add_argument(constants.COMMAND_READ,
                     metavar=constants.ARGS_FORMAT,
                     help=constants.Commands[constants.COMMAND_READ], required=False)
exp_com.add_argument(constants.COMMAND_RUN,
                     metavar=constants.ARGS_FORMAT,
                     help=constants.Commands[constants.COMMAND_RUN], required=False)

args = parser.parse_args()

# print(args)



# =====================================================================================================
# Processing operations
# =====================================================================================================
exp_config = read_config_file(os.path.join(get_script_root_path(), constants.PROPERTIES_FILE_NAME))
# exp_config = MyConfig()
init_env(os.path.join(get_script_root_path(), constants.REQUIREMENT_FILE_NAME))

if args.info:
    print_available_bv()
elif args.exp_config:
    show_config(exp_config)

elif args.read:
    bvs = args.read.split(',')
    for bv in bvs:
        repo_name, bug_id = split_src_num(bv)
        if repo_name and bug_id:
            repo, repo_path = get_repo_path(exp_config, repo_name, bug_id)
            p_path = read_d4j_bv(repo_path)

elif args.run:
    bvs = args.run.split(',')
    for bv in bvs:
        repo_name, bug_id = split_src_num(bv)
        if repo_name and bug_id:
            repo, repo_path = get_repo_path(exp_config, repo_name, bug_id)
            p_path = prepare_d4j_bv(exp_config, repo_path)
            run_(exp_config, p_path)

elif args.pre:
    bvs = args.pre.split(',')
    for bv in bvs:
        repo_name, bug_id = split_src_num(bv)
        if repo_name and bug_id:
            repo, repo_path = get_repo_path(exp_config, repo_name, bug_id)
            p_path = prepare_d4j_bv(exp_config, repo_path)
