#! /usr/bin/env python3
import os
import platform

PROPERTY_FILE_NAME = 'fixja.properties'
AN_PROPERTY_FILE_NAME = 'local_fixja.properties'
FIXJA_JAR = 'fixja-all.jar'
ARG_PROPERTY = '--FixjaSettingFile'

CLOUD_D4jDir = '/root/exp_tools/defects4j'
CLOUD_MsrLibDir = '/root/test/msr_test/libs'

p_JDKDir = 'JDKDir'
p_LogLevel = 'LogLevel'
p_ProjectRootDir = 'ProjectRootDir'
p_ProjectSourceDir = 'ProjectSourceDir'
p_ProjectOutputDir = 'ProjectOutputDir'
p_ProjectLib = 'ProjectLib'
p_ProjectTestSourceDir = 'ProjectTestSourceDir'
p_ProjectTestOutputDir = 'ProjectTestOutputDir'
p_ProjectTestsToInclude = 'ProjectTestsToInclude'
p_ProjectTestsToExclude = 'ProjectTestsToExclude'
p_ProjectExtraClasspath = 'ProjectExtraClasspath'
p_ProjectCompilationCommand = 'ProjectCompilationCommand'
p_ProjectExecutionCommand = 'ProjectExecutionCommand'
p_MethodToFix = 'MethodToFix'
p_Encoding = 'Encoding'
p_ExpToExclude = 'ExpToExclude'
p_TargetJavaVersion = 'TargetJavaVersion'

p_sep = ' ='
LINUX_PATH_SEP = '/'
LINUX_SEP = ':'
WIN_SEP = ';'

P_DEFAULT_LOGLEVEL = 'DEBUG'


def generate_new_properties_file_from_old(repository_path, my_config):
    old_repository_path = None
    old_property_file_path = os.path.join(repository_path, PROPERTY_FILE_NAME)
    another_property_file_path = os.path.join(repository_path, AN_PROPERTY_FILE_NAME)
    with open(old_property_file_path, 'r') as old_file:
        with open(another_property_file_path, 'w') as new_file:
            for line in old_file:
                if not line.startswith('LogFile'):
                    new_line = line
                    if p_JDKDir in line:
                        if my_config.JDKDir:
                            new_line = p_JDKDir + p_sep + my_config.JDKDir + os.linesep
                    elif p_LogLevel in line:
                        new_line = p_LogLevel + p_sep + P_DEFAULT_LOGLEVEL + os.linesep
                    elif p_ProjectRootDir in line and repository_path:
                        old_repository_path = line.split(p_sep)[1].strip()
                        new_line = p_ProjectRootDir + p_sep + repository_path + os.linesep
                    elif p_ProjectTestsToInclude in line:
                        old_test_to_include = line.split(p_sep)[1].strip()
                        new_test_to_include = check_test_list(old_test_to_include)
                        new_line = p_ProjectTestsToInclude + p_sep + new_test_to_include + os.linesep
                    elif p_ProjectTestsToExclude in line:
                        old_test_list = line.split(p_sep)[1].strip()
                        new_test_list = check_test_list(old_test_list)
                        new_line = p_ProjectTestsToExclude + p_sep + new_test_list + os.linesep
                    elif p_ProjectLib in line:
                        libs = line.split(p_sep)[1].split(os.pathsep)
                        if len(libs) <= 1:
                            libs = line.split(p_sep)[1].split(':')
                        new_libs = []
                        for lib in libs:
                            if 'junit-' in lib:
                                lib = my_config.get_junit_path()
                            elif old_repository_path in lib:
                                lib = os.path.relpath(lib, old_repository_path)
                            elif CLOUD_D4jDir in lib:
                                lib = os.path.join(my_config.get_d4j_path(), os.path.relpath(lib, CLOUD_D4jDir))
                            elif CLOUD_MsrLibDir in lib:
                                lib = os.path.join(my_config.get_lib_path(), os.path.relpath(lib, CLOUD_MsrLibDir))
                            elif lib.startswith('/tmp/'):
                                lib = lib[len('/tmp/'):len(lib)]
                                drive, repo_name = os.path.split(old_repository_path)
                                print(repo_name)
                                print(lib)
                                lib = lib[len(repo_name) + 1:len(lib)]

                            new_libs.append(lib)

                        has_hamcrest = False
                        for a_lib in new_libs:
                            if 'hamcrest' in a_lib:
                                has_hamcrest = True
                        if not has_hamcrest:
                            new_libs.insert(0,os.path.join(my_config.get_lib_path(), 'hamcrest-core-1.3.jar'))

                        new_libs = treatment_for_mockito(new_libs)
                        new_line = p_ProjectLib + p_sep + os.pathsep.join(new_libs) + os.linesep
                    new_line = path_normcase(new_line)
                    new_file.write(new_line)
    return another_property_file_path


def treatment_for_mockito(new_libs):
    hamcrest_core_11 = False
    hamcrest_all_13 = False
    for a_lib in new_libs:
        if 'build/classes/main' in a_lib:
            new_libs.remove(a_lib)
        if 'hamcrest-core-1.1.jar' in a_lib:
            hamcrest_core_11 = True
        if 'hamcrest-all-1.3.jar' in a_lib:
            hamcrest_all_13 = True
    if hamcrest_core_11 and hamcrest_all_13:
        for a_lib in new_libs:
            if 'hamcrest-all-1.3.jar' in a_lib:
                new_libs.remove(a_lib)
    return new_libs


def check_path_format(line):
    if str(platform.system()).lower().startswith('windows'):
        if LINUX_PATH_SEP in line:
            return line.replace(LINUX_PATH_SEP, os.path.sep)
    return line


def path_normcase(line):
    content = line.split(p_sep)
    if len(content) > 1 and (LINUX_PATH_SEP in content[1] or os.path.sep in content[1]):
        path = os.path.normpath(content[1].strip())
        if str(platform.system()).lower().startswith('windows'):
            path = path.replace(os.path.sep, "\\\\")
        new_ = content[0] + p_sep + path + os.linesep
        # print('new_line: ' + new_)
        return new_
    return line


def check_test_list(test_list):
    if LINUX_SEP in test_list:
        tests = test_list.strip().split(LINUX_SEP)
        return os.pathsep.join(tests)
    elif WIN_SEP in test_list:
        tests = test_list.strip().split(WIN_SEP)
        return os.pathsep.join(tests)
    return test_list
