import os
import urllib.request
import zipfile

import pip
import re
from pip.operations import freeze

from config import MyConfig
from constants import valid_bvs


def unzip(source_zip, target_dir):
    myzip = zipfile.ZipFile(source_zip)
    myzip.extractall(path=target_dir)
    myzip.close()


def zip_file(folder2zip, zip_file_path):
    f = zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED)
    current_dir = os.getcwd()
    os.chdir(folder2zip)

    for dir_path, dir_names, file_names in os.walk(folder2zip):
        for filename in file_names:
            f.write(os.path.join(dir_names, filename))
            # f.write(filename)
    f.close()
    os.chdir(current_dir)


def download_project(url, file_path):
    urllib.request.urlretrieve(url, file_path)


def read_config_file(config_path):  # Reading configuration for this script
    my_config = MyConfig()
    if os.path.isfile(config_path):
        with open(config_path, 'r') as file:
            for line in file:
                if line.startswith(my_config.p_JDKDir):
                    my_config.JDKDir = line[line.index('=') + 1:].strip()
    else:
        my_config.JDKDir = get_java8_home()
        write_config_file(config_path, my_config)
    return my_config


def write_config_file(config_path, my_config):
    if os.path.isfile(config_path):
        os.rmdir(config_path)
    with open(config_path, 'w') as file:
        file.write(my_config.p_JDKDir + '=' + my_config.JDKDir + os.linesep)


def get_java8_home():
    java_home = os.getenv("JAVA_HOME")
    while True:
        if java_home and '8' in java_home and 'jdk' in java_home.lower() and os.path.exists(java_home):
            return java_home
        java_home = input(
            'Fail to locate JDK8 automatically, \nplease input a correct JDK8 path:\n '
            '(e.g., For Mac OS: \"/Library/Java/JavaVirtualMachines/jdk1.8.0_31.jdk/Contents/Home\";\n'
            'For Windows: \"C:\Program Files\Java\jdk1.8.0_131\";\n'
            'For Linux: \"/usr/lib/jvm/java-8-openjdk-amd64\")\n')


def init_env(requirement_path):
    try:
        if not check_requirement(requirement_path):
            pip.main(["install", "-r", requirement_path])
        return True
    except Exception as detail:
        print(detail)
        print("Unable to install " + requirement_path +
              " using pip. Please read the instructions for manual installation.. Exiting")
        return False


def check_requirement(requirement_path):
    r_file = []
    r_current = []
    with open(requirement_path, 'r') as file:
        for line in file:
            r_file.append(line)
    x = freeze.freeze()
    for p in x:
        r_current.append(p)
    # print('r_current:' + str(r_current))
    # print('r_file:' + str(r_file))
    count = 0
    for one_r in r_file:
        if one_r.strip() in r_current:
            count += 1
    if count == len(r_file):
        return True
    return False


def split_src_num(candidate):
    r = re.compile("([a-zA-Z]+)([0-9]+)")
    m = r.match(candidate)
    repo_name = m.group(1)
    bug_id = m.group(2)
    return repo_name, bug_id


